@@ -31,11 +31,23 @@
                          target='_top')
     </PRE>"""
 
-__version__='$Revision: 1.9 $'[11:-2]
-
 from App.special_dtml import HTML
 
-MessageDialog = HTML("""
+
+class MessageDialogHTML(HTML):
+    """A special version of HTML which is always published as text/html
+    """
+    def __call__(self, *args, **kw):
+        class _HTMLString(str):
+            """A special string that will be published as text/html
+            """
+            def asHTML(self):
+                return self
+        return _HTMLString(
+            super(MessageDialogHTML, self).__call__(*args, **kw))
+
+
+MessageDialog = MessageDialogHTML("""
 <HTML>
 <HEAD>
 <TITLE>&dtml-title;</TITLE>
