@@ -17,12 +17,15 @@ import os
 import sys
 from logging import getLogger
 
+from six.moves.urllib.parse import urlparse
+
 import Products
 import transaction
 from AccessControl import ClassSecurityInfo
 from AccessControl.class_init import InitializeClass
 from AccessControl.Permission import ApplicationDefaultPermissions
 from AccessControl.Permissions import view_management_screens
+from AccessControl.tainted import TaintedString
 from Acquisition import aq_base
 from App import FactoryDispatcher
 from App.ApplicationManager import ApplicationManager
@@ -105,6 +108,49 @@ class Application(ApplicationDefaultPermissions, Folder.Folder, FindSupport):
 
     ZopeRedirect = Redirect
 
+    @security.protected(view_management_screens)
+    def getZMIMainFrameTarget(self, REQUEST):
+        """Utility method to get the right hand side ZMI frame source URL
+
+        For cases where JavaScript is disabled the ZMI uses a simple REQUEST
+        variable ``came_from`` to set the source URL for the right hand side
+        ZMI frame. Since this value can be manipulated by the user it must be
+        sanity-checked first.
+        """
+        parent_url = REQUEST['URL1']
+        default = '{}/manage_workspace'.format(parent_url)
+        came_from = REQUEST.get('came_from', None)
+
+        if not came_from:
+            return default
+
+        # When came_from contains suspicious code, it will not be a string,
+        # but an instance of AccessControl.tainted.TaintedString.
+        # Passing this to urlparse, gives:
+        # AttributeError: 'str' object has no attribute 'decode'
+        # This is good, but let's check explicitly.
+        if isinstance(came_from, TaintedString):
+            return default
+        try:
+            parsed_came_from = urlparse(came_from)
+        except AttributeError:
+            return default
+        parsed_parent_url = urlparse(parent_url)
+
+        # Only allow a passed-in ``came_from`` URL if it is local (just a path)
+        # or if the URL scheme and hostname are the same as our own
+        if (parsed_parent_url.scheme == parsed_came_from.scheme
+                and parsed_parent_url.netloc == parsed_came_from.netloc):
+            return came_from
+        if (not parsed_came_from.scheme and not parsed_came_from.netloc):
+            # This is only a path.  But some paths can be misinterpreted
+            # by browsers.
+            if parsed_came_from.path.startswith("//"):
+                return default
+            return came_from
+
+        return default
+
     def __bobo_traverse__(self, REQUEST, name=None):
         if name is None:
             # Make this more explicit, otherwise getattr(self, name)
@@ -295,8 +341,8 @@ class AppInitializer(object):
     def install_virtual_hosting(self):
         app = self.getApp()
         if 'virtual_hosting' not in app:
-            from Products.SiteAccess.VirtualHostMonster \
-                import VirtualHostMonster
+            from Products.SiteAccess.VirtualHostMonster import \
+                VirtualHostMonster
             any_vhm = [obj for obj in app.values()
                        if isinstance(obj, VirtualHostMonster)]
             if not any_vhm:
@@ -308,8 +354,8 @@ class AppInitializer(object):
     def install_root_view(self):
         app = self.getApp()
         if 'index_html' not in app:
-            from Products.PageTemplates.ZopePageTemplate \
-                import ZopePageTemplate
+            from Products.PageTemplates.ZopePageTemplate import \
+                ZopePageTemplate
             root_pt = ZopePageTemplate('index_html')
             root_pt.pt_setTitle(u'Auto-generated default page')
             app._setObject('index_html', root_pt)
