@@ -13,13 +13,19 @@
 """Image object
 """
 
+import os
 import struct
 from email.generator import _make_boundary
 from io import BytesIO
+from io import TextIOBase
+from mimetypes import guess_extension
+from tempfile import TemporaryFile
+from warnings import warn
 
 from six import PY2
 from six import binary_type
 from six import text_type
+from six.moves.urllib.parse import quote
 
 import ZPublisher.HTTPRequest
 from AccessControl.class_init import InitializeClass
@@ -27,9 +33,9 @@ from AccessControl.Permissions import change_images_and_files  # NOQA
 from AccessControl.Permissions import ftp_access
 from AccessControl.Permissions import view as View
 from AccessControl.Permissions import view_management_screens
+from AccessControl.Permissions import webdav_access
 from AccessControl.SecurityInfo import ClassSecurityInfo
 from Acquisition import Implicit
-from App.Common import rfc1123_date
 from App.special_dtml import DTMLFile
 from DateTime.DateTime import DateTime
 from OFS import bbb
@@ -43,6 +49,7 @@ from Persistence import Persistent
 from zExceptions import Redirect
 from zExceptions import ResourceLockedError
 from zope.contenttype import guess_content_type
+from zope.datetime import rfc1123_date
 from zope.event import notify
 from zope.interface import implementer
 from zope.lifecycleevent import ObjectCreatedEvent
@@ -57,6 +64,64 @@ except ImportError:  # PY2
     from cgi import escape
 
 
+def _get_list_from_env(name, default=None):
+    """Get list from environment variable.
+
+    Supports splitting on comma or white space.
+    Use the default as fallback only when the variable is not set.
+    So if the env variable is set to an empty string, this will ignore the
+    default and return an empty list.
+    """
+    value = os.environ.get(name)
+    if value is None:
+        return default or []
+    value = value.strip()
+    if "," in value:
+        return value.split(",")
+    return value.split()
+
+
+# We have one list for allowed, and one for disallowed inline mimetypes.
+# This is for security purposes.
+# By default we use the allowlist.  We give integrators the option to choose
+# the denylist via an environment variable.
+ALLOWED_INLINE_MIMETYPES = _get_list_from_env(
+    "ALLOWED_INLINE_MIMETYPES",
+    default=[
+        "image/gif",
+        # The mimetypes registry lists several for jpeg 2000:
+        "image/jp2",
+        "image/jpeg",
+        "image/jpeg2000-image",
+        "image/jpeg2000",
+        "image/jpx",
+        "image/png",
+        "image/webp",
+        "image/x-icon",
+        "image/x-jpeg2000-image",
+        "text/plain",
+        # By popular request we allow PDF:
+        "application/pdf",
+    ]
+)
+DISALLOWED_INLINE_MIMETYPES = _get_list_from_env(
+    "DISALLOWED_INLINE_MIMETYPES",
+    default=[
+        "application/javascript",
+        "application/x-javascript",
+        "text/javascript",
+        "text/html",
+        "image/svg+xml",
+        "image/svg+xml-compressed",
+    ]
+)
+try:
+    USE_DENYLIST = os.environ.get("OFS_IMAGE_USE_DENYLIST")
+    USE_DENYLIST = bool(int(USE_DENYLIST))
+except (ValueError, TypeError, AttributeError):
+    USE_DENYLIST = False
+
+
 manage_addFileForm = DTMLFile(
     'dtml/imageAdd',
     globals(),
@@ -116,6 +181,13 @@ class File(
     Cacheable
 ):
     """A File object is a content object for arbitrary files."""
+    # You can control which mimetypes may be shown inline
+    # and which must always be downloaded, for security reasons.
+    # Make the configuration available on the class.
+    # Then subclasses can override this.
+    allowed_inline_mimetypes = ALLOWED_INLINE_MIMETYPES
+    disallowed_inline_mimetypes = DISALLOWED_INLINE_MIMETYPES
+    use_denylist = USE_DENYLIST
 
     meta_type = 'File'
     zmi_icon = 'far fa-file-archive'
@@ -414,6 +486,19 @@ class File(
                         b'\r\n--' + boundary.encode('ascii') + b'--\r\n')
                     return True
 
+    def _should_force_download(self):
+        # If this returns True, the caller should set a
+        # Content-Disposition header with filename.
+        mimetype = self.content_type
+        if not mimetype:
+            return False
+        if self.use_denylist:
+            # We explicitly deny a few mimetypes, and allow the rest.
+            return mimetype in self.disallowed_inline_mimetypes
+        # Use the allowlist.
+        # We only explicitly allow a few mimetypes, and deny the rest.
+        return mimetype not in self.allowed_inline_mimetypes
+
     @security.protected(View)
     def index_html(self, REQUEST, RESPONSE):
         """
@@ -452,6 +537,24 @@ class File(
         RESPONSE.setHeader('Content-Length', self.size)
         RESPONSE.setHeader('Accept-Ranges', 'bytes')
 
+        if self._should_force_download():
+            # We need a filename, even a dummy one if needed.
+            filename = self.getId()
+            if "." not in filename:
+                # This either returns None or ".some_extension"
+                ext = guess_extension(self.content_type, strict=False)
+                if not ext:
+                    # image/svg+xml -> svg
+                    ext = "." + self.content_type.split("/")[-1].split("+")[0]
+                filename += ext
+            if not isinstance(filename, bytes):
+                filename = filename.encode("utf8")
+            filename = quote(filename)
+            RESPONSE.setHeader(
+                "Content-Disposition",
+                "attachment; filename*=UTF-8''{}".format(filename),
+            )
+
         if self.ZCacheable_isCachingEnabled():
             result = self.ZCacheable_get(default=None)
             if result is not None:
@@ -566,6 +669,16 @@ class File(
                 self, REQUEST, manage_tabs_message=msg)
 
     def _get_content_type(self, file, body, id, content_type=None):
+        """return content type or ``None``.
+
+        *file* usually is a ``FileUpload`` (like) instance; if this
+        specifies a content type, it is used. If *file*
+        is not ``FileUpload`` like, it is ignored and the
+        content type is guessed from the other parameters.
+
+        *body* is either a ``bytes`` or a ``Pdata`` instance
+        and assumed to be the *file* data.
+        """
         headers = getattr(file, 'headers', None)
         if headers and 'content-type' in headers:
             content_type = headers['content-type']
@@ -577,6 +690,13 @@ class File(
         return content_type
 
     def _read_data(self, file):
+        """return the data and size of *file* as tuple *data*, *size*.
+
+        *file* can be a ``bytes``, ``Pdata``, ``FileUpload`` or
+        (binary) file like instance.
+
+        For large files, *data* is a ``Pdata``, otherwise a ``bytes`` instance.
+        """
         import transaction
 
         n = 1 << 16
@@ -649,6 +769,47 @@ class File(
 
         return (_next, size)
 
+    @security.protected(change_images_and_files)
+    def PUT(self, REQUEST, RESPONSE):
+        """Handle HTTP PUT requests"""
+        self.dav__init(REQUEST, RESPONSE)
+        self.dav__simpleifhandler(REQUEST, RESPONSE, refresh=1)
+
+        type = REQUEST.get_header('content-type', None)
+        file = REQUEST['BODYFILE']
+
+        # Work around ``cgi`` bug
+        # ``cgi`` can turn the request body into a text file using
+        # the default encoding. ``File``, however, insists to work
+        # with bytes and binary files and forbids text.
+        # Convert back.
+        tfile = None
+        if isinstance(file, TextIOBase):  # ``cgi`` bug
+            if hasattr(file, "buffer"):
+                file = file.buffer  # underlying binary buffer
+            else:
+                from ZPublisher.HTTPRequest import default_encoding
+                tfile = TemporaryFile("wb+")
+                bufsize = 1 << 16
+                while True:
+                    data = file.read(bufsize)
+                    if not data:
+                        break
+                    tfile.write(data.encode(default_encoding))
+                file.seek(0, 0)
+                tfile.seek(0, 0)
+                file = tfile
+
+        data, size = self._read_data(file)
+        if tfile is not None:
+            tfile.close()
+        content_type = self._get_content_type(file, data, self.__name__,
+                                              type or self.content_type)
+        self.update_data(data, content_type, size)
+
+        RESPONSE.setStatus(204)
+        return RESPONSE
+
     @security.protected(View)
     def get_size(self):
         # Get the size of a file or image.
@@ -674,7 +835,10 @@ class File(
         if PY2:
             return str(self.data)
         else:
-            return self.data.decode(self._get_encoding())
+            if isinstance(self.data, Pdata):
+                return bytes(self.data).decode(self._get_encoding())
+            else:
+                return self.data.decode(self._get_encoding())
 
     def __bool__(self):
         return True
@@ -685,50 +849,40 @@ class File(
         data = bytes(self.data)
         return len(data)
 
+    @security.protected(webdav_access)
+    def manage_DAVget(self):
+        """Return body for WebDAV."""
+        RESPONSE = self.REQUEST.RESPONSE
+
+        if self.ZCacheable_isCachingEnabled():
+            result = self.ZCacheable_get(default=None)
+            if result is not None:
+                # We will always get None from RAMCacheManager but we will
+                # get something implementing the IStreamIterator interface
+                # from FileCacheManager.
+                # the content-length is required here by HTTPResponse.
+                RESPONSE.setHeader('Content-Length', self.size)
+                return result
+
+        data = self.data
+        if isinstance(data, binary_type):
+            RESPONSE.setBase(None)
+            return data
+
+        while data is not None:
+            RESPONSE.write(data.data)
+            data = data.next
+
+        return b''
+
     if bbb.HAS_ZSERVER:
-        @security.protected(change_images_and_files)
-        def PUT(self, REQUEST, RESPONSE):
-            """Handle HTTP PUT requests"""
-            self.dav__init(REQUEST, RESPONSE)
-            self.dav__simpleifhandler(REQUEST, RESPONSE, refresh=1)
-            type = REQUEST.get_header('content-type', None)
-
-            file = REQUEST['BODYFILE']
-
-            data, size = self._read_data(file)
-            content_type = self._get_content_type(file, data, self.__name__,
-                                                  type or self.content_type)
-            self.update_data(data, content_type, size)
-
-            RESPONSE.setStatus(204)
-            return RESPONSE
 
         @security.protected(ftp_access)
         def manage_FTPget(self):
             """Return body for ftp."""
-            RESPONSE = self.REQUEST.RESPONSE
-
-            if self.ZCacheable_isCachingEnabled():
-                result = self.ZCacheable_get(default=None)
-                if result is not None:
-                    # We will always get None from RAMCacheManager but we will
-                    # get something implementing the IStreamIterator interface
-                    # from FileCacheManager.
-                    # the content-length is required here by HTTPResponse,
-                    # even though FTP doesn't use it.
-                    RESPONSE.setHeader('Content-Length', self.size)
-                    return result
-
-            data = self.data
-            if isinstance(data, binary_type):
-                RESPONSE.setBase(None)
-                return data
-
-            while data is not None:
-                RESPONSE.write(data.data)
-                data = data.next
-
-            return b''
+            warn(u'manage_FTPget is deprecated and will be removed in Zope 5.',
+                 DeprecationWarning, stacklevel=2)
+            return self.manage_DAVget()
 
 
 InitializeClass(File)
@@ -881,16 +1035,9 @@ class Image(File):
     )
 
     manage_options = (
-        (
-            {
-                'label': 'Edit',
-                'action': 'manage_main',
-            },
-            {
-                'label': 'View',
-                'action': 'view_image_or_file',
-            }
-        )
+        ({'label': 'Edit', 'action': 'manage_main'},
+         {'label': 'View', 'action': 'view_image_or_file'})
+        + PropertyManager.manage_options
         + RoleManager.manage_options
         + Item_w__name__.manage_options
         + Cacheable.manage_options
