@@ -24,6 +24,7 @@ from doctest import REPORT_NDIFF
 from functools import partial
 from io import BytesIO
 
+from six import PY2
 from six import text_type
 
 import transaction
@@ -137,7 +138,8 @@ def http(request_string, handle_errors=True):
 
     This is used for HTTP doc tests.
     """
-    from six.moves.urllib.parse import unquote
+    from six.moves.urllib.parse import unquote_to_bytes
+
     from ZPublisher.HTTPRequest import WSGIRequest as Request
     from ZPublisher.HTTPResponse import WSGIResponse
     from ZPublisher.WSGIPublisher import publish_module
@@ -153,7 +155,6 @@ def http(request_string, handle_errors=True):
     command_line = request_string[:newline].rstrip()
     request_string = request_string[newline + 1:]
     method, path, protocol = command_line.split()
-    path = unquote(path)
 
     env = {
         'HTTP_HOST': 'localhost',
@@ -162,13 +163,14 @@ def http(request_string, handle_errors=True):
         'SERVER_PROTOCOL': protocol,
     }
 
-    p = path.split('?', 1)
-    if len(p) == 1:
-        env['PATH_INFO'] = p[0]
-    elif len(p) == 2:
-        [env['PATH_INFO'], env['QUERY_STRING']] = p
+    query = ''
+    if '?' in path:
+        path, query = path.split("?", 1)
+    if PY2:
+        env['PATH_INFO'] = unquote_to_bytes(path)
     else:
-        raise TypeError('')
+        env['PATH_INFO'] = unquote_to_bytes(path).decode('latin-1')
+    env['QUERY_STRING'] = query
 
     header_output = HTTPHeaderOutput(
         protocol, ('x-content-type-warning', 'x-powered-by'))
@@ -201,7 +203,6 @@ def http(request_string, handle_errors=True):
 
     outstream = BytesIO()
     response = WSGIResponse(stdout=outstream, stderr=sys.stderr)
-    request = Request(instream, env, response)
 
     env['wsgi.input'] = instream
     wsgi_headers = BytesIO()
@@ -214,7 +215,10 @@ def http(request_string, handle_errors=True):
         wsgi_headers.write(headers)
         wsgi_headers.write(b'\r\n\r\n')
 
-    publish = partial(publish_module, _request=request, _response=response)
+    publish = partial(
+        publish_module,
+        _request_factory=Request,
+        _response=response)
     if handle_errors:
         publish = HTTPExceptionHandler(publish)
 
