@@ -1,4 +1,5 @@
 ##############################################################################
+# coding: utf-8
 #
 # Copyright (c) 2002 Zope Foundation and Contributors.
 #
@@ -18,6 +19,7 @@ from io import BytesIO
 
 from six import PY2
 
+from AccessControl.tainted import TaintedString
 from AccessControl.tainted import should_be_tainted
 from zExceptions import NotFound
 from zope.component import getGlobalSiteManager
@@ -27,6 +29,7 @@ from zope.i18n.interfaces.locales import ILocale
 from zope.publisher.browser import BrowserLanguages
 from zope.publisher.interfaces.http import IHTTPRequest
 from zope.testing.cleanup import cleanUp
+from ZPublisher.HTTPRequest import FileUpload
 from ZPublisher.HTTPRequest import search_type
 from ZPublisher.interfaces import IXmlrpcChecker
 from ZPublisher.tests.testBaseRequest import TestRequestViewsBase
@@ -136,6 +139,7 @@ class HTTPRequestTests(unittest.TestCase, HTTPRequestFactoryMixin):
 
     def _processInputs(self, inputs):
         from six.moves.urllib.parse import quote_plus
+
         # Have the inputs processed, and return a HTTPRequest object
         # holding the result.
         # inputs is expected to be a list of (key, value) tuples, no CGI
@@ -162,8 +166,8 @@ class HTTPRequestTests(unittest.TestCase, HTTPRequestFactoryMixin):
         # when one is found.
         # Also raises an Assertion if a string which *should* have been
         # tainted is found, or when a tainted string is not deemed dangerous.
-        from ZPublisher.HTTPRequest import record
         from AccessControl.tainted import TaintedString
+        from ZPublisher.HTTPRequest import record
 
         retval = 0
 
@@ -883,7 +887,6 @@ class HTTPRequestTests(unittest.TestCase, HTTPRequestFactoryMixin):
         self.assertEqual(password_x, password)
 
     def test__authUserPW_with_embedded_colon(self):
-        # http://www.zope.org/Collectors/Zope/2039
         user_id = 'user'
         password = 'embedded:colon'
         auth_header = basic_auth_encode(user_id, password)
@@ -896,8 +899,22 @@ class HTTPRequestTests(unittest.TestCase, HTTPRequestFactoryMixin):
         self.assertEqual(user_id_x, user_id)
         self.assertEqual(password_x, password)
 
+    def test__authUserPW_non_ascii(self):
+        user_id = 'usèr'
+        password = 'pàssword'
+        auth_header = basic_auth_encode(user_id, password)
+
+        environ = {'HTTP_AUTHORIZATION': auth_header}
+        request = self._makeOne(environ=environ)
+
+        user_id_x, password_x = request._authUserPW()
+
+        self.assertEqual(user_id_x, user_id)
+        self.assertEqual(password_x, password)
+
     def test_debug_not_in_qs_still_gets_attr(self):
         from zope.publisher.base import DebugFlags
+
         # when accessing request.debug we will see the DebugFlags instance
         request = self._makeOne()
         self.assertIsInstance(request.debug, DebugFlags)
@@ -1150,8 +1167,8 @@ class HTTPRequestTests(unittest.TestCase, HTTPRequestFactoryMixin):
         self.assertIsInstance(clone, SubRequest)
 
     def test_clone_preserves_direct_interfaces(self):
-        from zope.interface import directlyProvides
         from zope.interface import Interface
+        from zope.interface import directlyProvides
 
         class IFoo(Interface):
             pass
@@ -1162,6 +1179,13 @@ class HTTPRequestTests(unittest.TestCase, HTTPRequestFactoryMixin):
         self.assertTrue(IFoo.providedBy(clone))
 
     def test_resolve_url_doesnt_send_endrequestevent(self):
+        # The following imports are necessary:
+        #  They happen implicitely in `request.resolve_url`
+        #  They creates `zope.schema` events
+        # Doing them here avoids those unrelated events
+        import OFS.PropertyManager  # noqa: F401
+        import OFS.SimpleItem  # noqa: F401
+        #
         import zope.event
         events = []
         zope.event.subscribers.append(events.append)
@@ -1299,6 +1323,19 @@ class HTTPRequestTests(unittest.TestCase, HTTPRequestFactoryMixin):
         req = self._makeOne(environ=env)
         self.assertEqual(req['SERVER_URL'], 'https://myhost')
 
+    def test_issue_1095(self):
+        body = TEST_ISSUE_1095_DATA
+        env = self._makePostEnviron(body)
+        req = self._makeOne(BytesIO(body), env)
+        req.processInputs()
+        r = req["r"]
+        self.assertEqual(len(r), 2)
+        self.assertIsInstance(r[0].x, FileUpload)
+        self.assertIsInstance(r[1].x, str)
+        r = req.taintedform["r"]
+        self.assertIsInstance(r[0].x, FileUpload)
+        self.assertIsInstance(r[1].x, TaintedString)
+
 
 class TestHTTPRequestZope3Views(TestRequestViewsBase):
 
@@ -1379,3 +1416,19 @@ Content-Disposition: form-data; name="largefile"; filename="largefile"
 Content-Type: application/octet-stream
 
 test ''' + (b'test' * 1000) + b'\n\n'
+
+TEST_ISSUE_1095_DATA = b'''
+--12345
+Content-Disposition: form-data; name="r.x:records"; filename="fn"
+Content-Type: application/octet-stream
+
+test
+
+--12345
+Content-Disposition: form-data; name="r.x:records"
+Content-Type: text/html
+
+<body>abc</body>
+
+--12345--
+'''
