@@ -1,6 +1,6 @@
 ##############################################################################
 #
-# Copyright (c) 2003 Zope Corporation and Contributors.
+# Copyright (c) 2003 Zope Foundation and Contributors.
 # All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
@@ -11,74 +11,67 @@
 # FOR A PARTICULAR PURPOSE.
 #
 ##############################################################################
-
 """Datatypes for the Zope schema for use with ZConfig."""
 
+import io
 import os
-from UserDict import UserDict
+import traceback
+
+from six import PY2
+from six import text_type
+from six.moves import UserDict
 
-from ZConfig.components.logger import logger
 from ZODB.config import ZODBDatabase
+from zope.deferredimport import deprecated
+
+
+# BBB Zope 5.0
+_prefix = 'ZServer.Zope2.Startup.datatypes:'
+deprecated(
+    'Please import from ZServer.Zope2.Startup.datatypes.',
+    cgi_environment=_prefix + 'cgi_environment',
+    LoggerFactory=_prefix + 'LoggerFactory',
+    dns_resolver=_prefix + 'dns_resolver',
+    python_dotted_path=_prefix + 'python_dotted_path',
+    zdaemonEnvironDict=_prefix + 'zdaemonEnvironDict',
+    root_config=_prefix + 'root_config',
+    minimalClassFactory=_prefix + 'minimalClassFactory',
+)
 
-# generic datatypes
 
 def security_policy_implementation(value):
     value = value.upper()
     ok = ('PYTHON', 'C')
     if value not in ok:
-        raise ValueError, (
-            "security-policy-implementation must be one of %s" % repr(ok)
-            )
+        raise ValueError(
+            "security-policy-implementation must be one of %r" % ok)
     return value
 
+
 def datetime_format(value):
     value = value.lower()
     ok = ('us', 'international')
     if value not in ok:
-        raise ValueError, (
-            "datetime-format must be one of %r" % repr(ok)
-            )
+        raise ValueError("datetime-format must be one of %r" % ok)
     return value
 
-def cgi_environment(section):
+
+def environment(section):
     return section.environ
 
-# Datatype for the access and trace logs
-# (the loghandler datatypes come from the zLOG package)
-
-class LoggerFactory(logger.LoggerFactory):
-    """
-    A factory used to create loggers while delaying actual logger
-    instance construction.  We need to do this because we may want to
-    reference a logger before actually instantiating it (for example,
-    to allow the app time to set an effective user).  An instance of
-    this wrapper is a callable which, when called, returns a logger
-    object.
-    """
-    def __init__(self, section):
-        section.name = section.getSectionName()
-        section.propagate = False
-        logger.LoggerFactory.__init__(self, section)
-
-# DNS resolver
-
-def dns_resolver(hostname):
-    from ZServer.medusa import resolver
-    return resolver.caching_resolver(hostname)
-
-# mount-point definition
 
 def mount_point(value):
+    # mount-point definition
     if not value:
-        raise ValueError, 'mount-point must not be empty'
+        raise ValueError('mount-point must not be empty')
     if not value.startswith('/'):
-        raise ValueError, ("mount-point '%s' is invalid: mount points must "
-                           "begin with a slash" % value)
+        raise ValueError("mount-point '%s' is invalid: mount points must "
+                         "begin with a slash" % value)
     return value
 
-# A datatype that converts a Python dotted-path-name to an object
 
 def importable_name(name):
+    # A datatype that converts a Python dotted-path-name to an object
     try:
         components = name.split('.')
         start = components[0]
@@ -94,56 +87,36 @@ def importable_name(name):
                 package = __import__(n, g, g, component)
         return package
     except ImportError:
-
-        import traceback, cStringIO
-        IO = cStringIO.StringIO()
+        IO = io.StringIO()
         traceback.print_exc(file=IO)
         raise ValueError(
-            'The object named by "%s" could not be imported\n%s' %  (name, IO.getvalue()))
-
-# A datatype that ensures that a dotted path name can be resolved but
-# returns the name instead of the object
-
-def python_dotted_path(name):
-    ob = importable_name(name) # will fail in course
-    return name
+            'The object named by "%s" could not be imported\n%s' % (
+                name, IO.getvalue()))
 
 
-class zdaemonEnvironDict(UserDict):
+class ZDaemonEnvironDict(UserDict):
     # zdaemon 2 expects to use a 'mapping' attribute of the environ object.
+
     @property
     def mapping(self):
         return self.data
 
-# Datatype for the root configuration object
-# (adds the softwarehome and zopehome fields; default values for some
-#  computed paths, configures the dbtab)
 
-def root_config(section):
+def root_wsgi_config(section):
     from ZConfig import ConfigurationError
     from ZConfig.matcher import SectionValue
-    here = os.path.dirname(os.path.abspath(__file__))
-    swhome = os.path.dirname(os.path.dirname(here))
-    section.softwarehome = swhome
-    section.zopehome = os.path.dirname(os.path.dirname(swhome))
     if section.environment is None:
-        section.environment = zdaemonEnvironDict()
-    if section.cgi_environment is None:
-        section.cgi_environment = zdaemonEnvironDict()
+        section.environment = ZDaemonEnvironDict()
     if section.clienthome is None:
         section.clienthome = os.path.join(section.instancehome, "var")
-    # set up defaults for pid_filename and lock_filename if they're
-    # not in the config
-    if section.pid_filename is None:
-        section.pid_filename = os.path.join(section.clienthome, 'Z2.pid')
-    if section.lock_filename is None:
-        section.lock_filename = os.path.join(section.clienthome, 'Z2.lock')
+    if getattr(section, 'pid_filename', None) is None:
+        section.pid_filename = os.path.join(section.clienthome, 'Z4.pid')
 
     if not section.databases:
         section.databases = []
 
-    mount_factories = {} # { name -> factory}
-    mount_points = {} # { virtual path -> name }
+    mount_factories = {}  # { name -> factory}
+    mount_points = {}  # { virtual path -> name }
     dup_err = ('Invalid configuration: ZODB databases named "%s" and "%s" are '
                'both configured to use the same mount point, named "%s"')
 
@@ -152,7 +125,7 @@ def root_config(section):
         name = database.config.getSectionName()
         mount_factories[name] = database
         for point in points:
-            if mount_points.has_key(point):
+            if point in mount_points:
                 raise ConfigurationError(dup_err % (mount_points[point],
                                                     name, point))
             mount_points[point] = name
@@ -173,6 +146,7 @@ def root_config(section):
 
     return section
 
+
 class ZopeDatabase(ZODBDatabase):
     """ A ZODB database datatype that can handle an extended set of
     attributes for use by DBTab """
@@ -188,8 +162,6 @@ class ZopeDatabase(ZODBDatabase):
             DB.klass = self.config.connection_class
         if self.config.class_factory is not None:
             DB.classFactory = self.config.class_factory
-        from ZODB.ActivityMonitor import ActivityMonitor
-        DB.setActivityMonitor(ActivityMonitor())
         return DB
 
     def getName(self):
@@ -198,6 +170,8 @@ class ZopeDatabase(ZODBDatabase):
     def computeMountPaths(self):
         mps = []
         for part in self.config.mount_points:
+            if PY2 and isinstance(part, text_type):
+                part = part.encode()
             real_root = None
             if ':' in part:
                 # 'virtual_path:real_path'
@@ -228,19 +202,29 @@ class ZopeDatabase(ZODBDatabase):
                     container_class = 'OFS.Folder.Folder'
                 return (real_root, real_path, container_class)
         raise LookupError('Nothing known about mount path %s' % mount_path)
-    
+
+
 def default_zpublisher_encoding(value):
     # This is a bit clunky but necessary :-(
     # These modules are imported during the configuration process
     # so a module-level call to getConfiguration in any of them
     # results in getting config data structure without the necessary
     # value in it.
-    from ZPublisher import Converters, HTTPRequest, HTTPResponse
+    if PY2:
+        # unicode is not acceptable as encoding in HTTP headers:
+        value = str(value)
+    from ZPublisher import Converters
+    from ZPublisher import HTTPRequest
+    from ZPublisher import HTTPResponse
     Converters.default_encoding = value
     HTTPRequest.default_encoding = value
+    HTTPRequest.HTTPRequest.charset = value
     HTTPResponse.default_encoding = value
+    HTTPResponse.HTTPBaseResponse.charset = value
+    return value
 
-class DBTab:
+
+class DBTab(object):
     """A Zope database configuration, similar in purpose to /etc/fstab.
     """
 
@@ -252,19 +236,16 @@ class DBTab:
     def listMountPaths(self):
         """Returns a sequence of (virtual_mount_path, database_name).
         """
-        return self.mount_paths.items()
-
+        return list(self.mount_paths.items())
 
     def listDatabaseNames(self):
         """Returns a sequence of names.
         """
-        return self.db_factories.keys()
-
+        return list(self.db_factories.keys())
 
     def hasDatabase(self, name):
         """Returns true if name is the name of a configured database."""
-        return self.db_factories.has_key(name)
-
+        return name in self.db_factories
 
     def _mountPathError(self, mount_path):
         from ZConfig import ConfigurationError
@@ -290,7 +271,7 @@ class DBTab:
     def getDatabaseFactory(self, mount_path=None, name=None):
         if name is None:
             name = self.getName(mount_path)
-        if not self.db_factories.has_key(name):
+        if name not in self.db_factories:
             raise KeyError('%s is not a configured database' % repr(name))
         return self.db_factories[name]
 
@@ -300,28 +281,9 @@ class DBTab:
             self._mountPathError(mount_path)
         return name
 
-# class factories (potentially) used by the class-factory parameter in
-# zopeschema.xml
 
-def minimalClassFactory(jar, module, name,
-                        _silly=('__doc__',), _globals={},
-                        ):
-    """Minimal class factory.
-
-    If any class is not found, this class factory will propagate
-    the exception to the application, unlike the other class factories.
+def simpleClassFactory(jar, module, name, _silly=('__doc__',), _globals={}):
+    """Class factory.
     """
     m = __import__(module, _globals, _globals, _silly)
     return getattr(m, name)
-
-def simpleClassFactory(jar, module, name,
-                       _silly=('__doc__',), _globals={},
-                       ):
-    """Class factory.
-    """
-    import OFS.Uninstalled
-    try:
-        m = __import__(module, _globals, _globals, _silly)
-        return getattr(m, name)
-    except:
-        return OFS.Uninstalled.Broken(jar, None, (module, name))
